/* 
 * File cpp della classe FeaturesIstogramma
 */

#include "FeaturesIstogramma.h"
#include "FileConfigurazione.h"

using namespace Eigen;
using namespace cv;
using namespace std;

static const int kNumBins = 16;
static const int kNumLevels = 4;

FeaturesIstogramma::FeaturesIstogramma(const FileConfigurazione & conf)
{
	int nc = 0;
	for (int i = 0; i < kNumLevels; ++i)
	{
		//nc += 1 << 2*i;
		nc += (i+1)*(i+1);
	}
	SetCount(kNumBins*nc);
	cout << "histogram bins: " << GetCount() << endl;
}

void FeaturesIstogramma::AggiornaFeatureVector(const CampioneSingolo &s)
{
	IntRect rect = s.GetROI(); // note this truncates to integers
	//cv::RettBoundingBox roi(rect.XMin(), rect.YMin(), rect.Width(), rect.Height());
	//cv::resize(s.GetImage().GetImmagine(0)(roi), m_patchImage, m_patchImage.size());
	
	m_featVec.setZero();
	VectorXd istogramma(kNumBins);
	
	int histind = 0;
	for (int il = 0; il < kNumLevels; ++il)
	{
		int nc = il+1;
		float w = s.GetROI().Width()/nc;
		float h = s.GetROI().Height()/nc;
		FloatRect cell(0.f, 0.f, w, h);
		for (int iy = 0; iy < nc; ++iy)
		{
			cell.SetYMin(s.GetROI().YMin()+iy*h);
			for (int ix = 0; ix < nc; ++ix)
			{
				cell.SetXMin(s.GetROI().XMin()+ix*w);
				s.GetImage().Istogramma(cell, istogramma);
				m_featVec.segment(histind*kNumBins, kNumBins) = istogramma;
				++histind;
			}
		}
	}
	m_featVec /= histind;
}
