/*
 * Cpp file della classe FileConfigurazione
 */

#include "FileConfigurazione.h"

#include <fstream>
#include <iostream>
#include <sstream>

using namespace std;

FileConfigurazione::FileConfigurazione(const std::string& path)
{
	SettaValoriDefault();
	//TODO: rimuovere valori hardcoded ed inserire parser
	seed = 0;
	quietMode = 0;
	debugMode = 0;
	sequenceBasePath = "sequences";
	sequenceName = "girl";
	frameWidth = 640;
	frameHeight = 480;
	searchRadius = 30;
	svmC = 100;
	svmBudgetSize = 100;
	CoppiaFeatureKernel fkp;
	fkp.feature = kFeatureTipoHaar;
	fkp.kernel = kKernelTipoGaussiano;
	fkp.params.push_back(0.2);
	TipologiaFeatures.push_back(fkp);
	// fine valori hardcoded
	ifstream f(path.c_str());
	if (!f)
	{
		cout << "error: could not load config file: " << path << endl;
		return;
	}

	string line, name, tmp;
	while (getline(f, line))
	{
		istringstream iss(line);
		iss >> name >> tmp;

		// skip invalid lines and comments
		if (iss.fail() || tmp != "=" || name[0] == '#') continue;

		if      (name == "seed") iss >> seed;
		else if (name == "quietMode") iss >> quietMode;
		else if (name == "debugMode") iss >> debugMode;
		else if (name == "sequenceBasePath") iss >> sequenceBasePath;
		else if (name == "sequenceName") iss >> sequenceName;
		else if (name == "resultsPath") iss >> resultsPath;
		else if (name == "frameWidth") iss >> frameWidth;
		else if (name == "frameHeight") iss >> frameHeight;
		else if (name == "seed") iss >> seed;
		else if (name == "searchRadius") iss >> searchRadius;
		else if (name == "svmC") iss >> svmC;
		else if (name == "svmBudgetSize") iss >> svmBudgetSize;
		else if (name == "feature")
		{
			string featureName, kernelName;
			double param;
			iss >> featureName >> kernelName >> param;

			CoppiaFeatureKernel fkp;

			if      (featureName == NomeFeature(kFeatureTipoHaar)) fkp.feature = kFeatureTipoHaar;
			else if (featureName == NomeFeature(kFeatureTipoRaw)) fkp.feature = kFeatureTipoRaw;
			else if (featureName == NomeFeature(kFeatureTipoIstogramma)) fkp.feature = kFeatureTipoIstogramma;
			else
			{
				cout << "error: unrecognised feature: " << featureName << endl;
				continue;
			}

			if      (kernelName == NomeKernel(kKernelTipoLineare)) fkp.kernel = kKernelTipoLineare;
			else if (kernelName == NomeKernel(kKernelTipoIntersection)) fkp.kernel = kKernelTipoIntersection;
			else if (kernelName == NomeKernel(kKernelTipoChi2)) fkp.kernel = kKernelTipoChi2;
			else if (kernelName == NomeKernel(kKernelTipoGaussiano))
			{
				if (iss.fail())
				{
					cout << "error: gaussian kernel requires a parameter (sigma)" << endl;
					continue;
				}
				fkp.kernel = kKernelTipoGaussiano;
				fkp.params.push_back(param);
			}
			else
			{
				cout << "error: unrecognised kernel: " << kernelName << endl;
				continue;
			}

			TipologiaFeatures.push_back(fkp);
		}
	}
}

void FileConfigurazione::SettaValoriDefault()
{

	quietMode = false;
	debugMode = false;

	sequenceBasePath = "";
	sequenceName = "";
	resultsPath = "";

	frameWidth = 320;
	frameHeight = 240;

	seed = 0;
	searchRadius = 30;
	svmC = 1.0;
	svmBudgetSize = 0;

	TipologiaFeatures.clear();
}

std::string FileConfigurazione::NomeFeature(TipoFeature f)
{
	switch (f)
	{
	case kFeatureTipoRaw:
		return "raw";
	case kFeatureTipoHaar:
		return "haar";
	case kFeatureTipoIstogramma:
		return "histogram";
	default:
		return "";
	}
}

std::string FileConfigurazione::NomeKernel(TipoKernel k)
{
	switch (k)
	{
	case kKernelTipoLineare:
		return "linear";
	case kKernelTipoGaussiano:
		return "gaussian";
	case kKernelTipoIntersection:
		return "intersection";
	case kKernelTipoChi2:
		return "chi2";
	default:
		return "";
	}
}

ostream& operator<< (ostream& out, const FileConfigurazione & conf)
{
	out << "config:" << endl;
	out << "  quietMode          = " << conf.quietMode << endl;
	out << "  debugMode          = " << conf.debugMode << endl;
	out << "  sequenceBasePath   = " << conf.sequenceBasePath << endl;
	out << "  sequenceName       = " << conf.sequenceName << endl;
	out << "  resultsPath        = " << conf.resultsPath << endl;
	out << "  frameWidth         = " << conf.frameWidth << endl;
	out << "  frameHeight        = " << conf.frameHeight << endl;
	out << "  seed               = " << conf.seed << endl;
	out << "  searchRadius       = " << conf.searchRadius << endl;
	out << "  svmC               = " << conf.svmC << endl;
	out << "  svmBudgetSize      = " << conf.svmBudgetSize << endl;

	for (int i = 0; i < (int)conf.TipologiaFeatures.size(); ++i)
	{
		out << "  feature " << i << endl;
		out << "    feature: " << FileConfigurazione::NomeFeature(conf.TipologiaFeatures[i].feature) << endl;
		out << "    kernel:  " << FileConfigurazione::NomeKernel(conf.TipologiaFeatures[i].kernel) <<endl;
		if (conf.TipologiaFeatures[i].params.size() > 0)
		{
			out << "    params: ";
			for (int j = 0; j < (int)conf.TipologiaFeatures[i].params.size(); ++j)
			{
				out << " " << conf.TipologiaFeatures[i].params[j];
			}
			out << endl;
		}
	}

	return out;
}
