/* 
 * File cpp della classe Immagine
 */

#include "Immagine.h"

#include <cassert>

#include <opencv2/highgui/highgui.hpp>
#include <opencv2/imgproc/imgproc.hpp>
using namespace std;
using namespace cv;

static const int kNumBins = 16;

Immagine::Immagine(const Mat& image, bool computeIntegral, bool computeIntegralHist, bool colour) :
	Canali(colour ? 3 : 1),
	Mio_Rect(0, 0, image.cols, image.rows)
{	
	for (int i = 0; i < Canali; ++i)
	{
		Immagini.push_back(Mat(image.rows, image.cols, CV_8UC1));
		if (computeIntegral) ImmaginiIntegrali.push_back(Mat(image.rows+1, image.cols+1, CV_32SC1));
		if (computeIntegralHist)
		{
			for (int j = 0; j < kNumBins; ++j)
			{
				IstrogrammiImmaginiIntegrali.push_back(Mat(image.rows+1, image.cols+1, CV_32SC1));
			}
		}
	}
		
	if (colour)
	{
		assert(image.channels() == 3);
		split(image, Immagini);
	}
	else
	{
		assert(image.channels() == 1 || image.channels() == 3);
		if (image.channels() == 3)
		{
			cv::cvtColor(image, Immagini[0], CV_RGB2GRAY);
		}
		else if (image.channels() == 1)
		{
			image.copyTo(Immagini[0]);
		}
	}
	
	if (computeIntegral)
	{
		for (int i = 0; i < Canali; ++i)
		{
			//equalizeHist(m_images[i], m_images[i]);
			integral(Immagini[i], ImmaginiIntegrali[i]);
		}
	}
	
	if (computeIntegralHist)
	{
		Mat tmp(image.rows, image.cols, CV_8UC1);
		tmp.setTo(0);
		for (int j = 0; j < kNumBins; ++j)
		{
			for (int y = 0; y < image.rows; ++y)
			{
				const uchar* src = Immagini[0].ptr(y);
				uchar* dst = tmp.ptr(y);
				for (int x = 0; x < image.cols; ++x)
				{
					int bin = (int)(((float)*src/256)*kNumBins);
					*dst = (bin == j) ? 1 : 0;
					++src;
					++dst;
				}
			}
			
			integral(tmp, IstrogrammiImmaginiIntegrali[j]);
		}
	}
}

int Immagine::Somma(const IntRect &rRect, int channel) const
{
	assert(rRect.XMin() >= 0 && rRect.YMin() >= 0 && rRect.XMax() <= Immagini[0].cols && rRect.YMax() <= Immagini[0].rows);
	return ImmaginiIntegrali[channel].at<int>(rRect.YMin(), rRect.XMin()) +
			ImmaginiIntegrali[channel].at<int>(rRect.YMax(), rRect.XMax()) -
			ImmaginiIntegrali[channel].at<int>(rRect.YMax(), rRect.XMin()) -
			ImmaginiIntegrali[channel].at<int>(rRect.YMin(), rRect.XMax());
}

void Immagine::Istogramma(const IntRect &rRect, Eigen::VectorXd &h) const
{
	assert(rRect.XMin() >= 0 && rRect.YMin() >= 0 && rRect.XMax() <= Immagini[0].cols && rRect.YMax() <= Immagini[0].rows);
	int norm = rRect.Area();
	for (int i = 0; i < kNumBins; ++i)
	{
		int sum = IstrogrammiImmaginiIntegrali[i].at<int>(rRect.YMin(), rRect.XMin()) +
			IstrogrammiImmaginiIntegrali[i].at<int>(rRect.YMax(), rRect.XMax()) -
			IstrogrammiImmaginiIntegrali[i].at<int>(rRect.YMax(), rRect.XMin()) -
			IstrogrammiImmaginiIntegrali[i].at<int>(rRect.YMin(), rRect.XMax());
		h[i] = (float)sum/norm;
	}
}
