/* 
 * Struck: Structured Output Tracking with Kernels
 * 
 * Code to accompany the paper:
 *   Struck: Structured Output Tracking with Kernels
 *   Sam Hare, Amir Saffari, Philip H. S. Torr
 *   International Conference on Computer Vision (ICCV), 2011
 * 
 * Copyright (C) 2011 Sam Hare, Oxford Brookes University, Oxford, UK
 * 
 * This file is part of Struck.
 * 
 * Struck is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Struck is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Struck.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

#ifndef LARANK_H
#define LARANK_H

#include "RettBoundingBox.h"
#include "CampioneSingolo.h"

#include <vector>
#include <../Eigen/Eigen/Core>

#include <opencv2/opencv.hpp>

class FileConfigurazione;
class Features;
class Kernel;

class OnlineLearning
{
public:
	OnlineLearning(const FileConfigurazione & conf, const Features& features, const Kernel& kernel);
	~OnlineLearning();
	
	virtual void Eval(const MultiCampione & x, std::vector<double>& results);
	virtual void Aggiornamento(const MultiCampione &campione, int y);
	
	virtual void Debug();

private:

	struct SupportPattern
	{
		std::vector<Eigen::VectorXd> x;
		std::vector<FloatRect> yv;
		std::vector<cv::Mat> images;
		int y;
		int refCount;
	};

	struct SupportVector
	{
		SupportPattern* x;
		int y;
		double b;
		double g;
		cv::Mat image;
	};
	
	const FileConfigurazione &Mia_Configurazione;
	const Features&Mie_Features;
	const Kernel&Mio_Kernel;
	
	std::vector<SupportPattern*> Pattern_Supporto;
	std::vector<SupportVector*> Vettori_Supporto;

	cv::Mat ImmagineDebug;
	
	double m_C;
	Eigen::MatrixXd m_K;

	inline double Loss(const FloatRect& y1, const FloatRect& y2) const //Valuto la funzione di perdita -- varie definizioni
	{
		return 1.0-y1.Overlap(y2);
		// squared distance loss
		//double dx = y1.XMin()-y2.XMin();
		//double dy = y1.YMin()-y2.YMin();
		//return dx*dx+dy*dy;
	}
	
	double ComputeDual() const;

	void SMOStep(int ipos, int ineg);
	std::pair<int, double> MinGradiente(int ind);
	void ProcessNew(int ind);
	void Reprocess();
	void ProcessOld();
	void Ottimizza();

	int AggiungiSupportVector(SupportPattern *x, int y, double g);
	void RimuoviSupportVector(int ind);
	void RemoveSupportVectors(int ind1, int ind2);
	void ScambiaSupportVectors(int ind1, int ind2);
	
	void BudgetMaintenance();
	void BudgetMaintenanceRemove();

	double Valutazione(const Eigen::VectorXd &x, const FloatRect &y) const;
	void AggiornaImmagineDebug();
};

#endif
