/*
 * Header File della classe Rect
 */
#ifndef RECT_H
#define RECT_H

#include <iostream>
#include <algorithm>

template <typename T>
class RettBoundingBox
{
public:
	RettBoundingBox() :
		Mio_xMin(0),
		Mio_yMin(0),
		Mia_width(0),
		Mia_height(0)
	{
	}
	
	RettBoundingBox(T xMin, T yMin, T width, T height) :
		Mio_xMin(xMin),
		Mio_yMin(yMin),
		Mia_width(width),
		Mia_height(height)
	{
	}
	
	template <typename T2>
	RettBoundingBox(const RettBoundingBox<T2>& rOther) :
		Mio_xMin((T)rOther.XMin()),
		Mio_yMin((T)rOther.YMin()),
		Mia_width((T)rOther.Width()),
		Mia_height((T)rOther.Height())
	{
	}

	inline void Set(T xMin, T yMin, T width, T height)
	{
		Mio_xMin = xMin;
		Mio_yMin = yMin;
		Mia_width = width;
		Mia_height = height;
	}
	
	inline T XMin() const { return Mio_xMin; }
	inline void SetXMin(T val) { Mio_xMin = val; }
	inline T YMin() const { return Mio_yMin; }
	inline void SetYMin(T val) { Mio_yMin = val; }
	inline T Width() const { return Mia_width; }
	inline void SetWidth(T val) { Mia_width = val; }
	inline T Height() const { return Mia_height; }
	inline void SetHeight(T val) { Mia_height = val; }
	
	inline void Translate(T x, T y) { Mio_xMin += x; Mio_yMin += y; }

	inline T XMax() const { return Mio_xMin + Mia_width; }
	inline T YMax() const { return Mio_yMin + Mia_height; }
	inline float XCentre() const { return (float) Mio_xMin + (float) Mia_width / 2; }
	inline float YCentre() const { return (float) Mio_yMin + (float) Mia_height / 2; }
	inline T Area() const { return Mia_width * Mia_height; }
	
	template <typename T2>
	friend std::ostream& operator <<(std::ostream &rOS, const RettBoundingBox<T2>& rRect);
	
	template <typename T2>
	float Overlap(const RettBoundingBox<T2>& rOther) const;
	
	template <typename T2>
	bool Interno(const RettBoundingBox<T2> &rOther) const;

private:
	T Mio_xMin;
	T Mio_yMin;
	T Mia_width;
	T Mia_height;
};

typedef RettBoundingBox<int> IntRect;
typedef RettBoundingBox<float> FloatRect;

template <typename T>
std::ostream& operator <<(std::ostream &rOS, const RettBoundingBox<T>& rRect)
{
	rOS << "[origin: (" << rRect.Mio_xMin << ", " << rRect.Mio_yMin << ") size: (" << rRect.Mia_width << ", " << rRect.Mia_height << ")]";
	return rOS;
}

template <typename T>
template <typename T2>
float RettBoundingBox<T>::Overlap(const RettBoundingBox<T2>& rOther) const
{
	float x0 = std::max((float)XMin(), (float)rOther.XMin());
	float x1 = std::min((float)XMax(), (float)rOther.XMax());
	float y0 = std::max((float)YMin(), (float)rOther.YMin());
	float y1 = std::min((float)YMax(), (float)rOther.YMax());
	
	if (x0 >= x1 || y0 >= y1) return 0.f;
	
	float areaInt = (x1-x0)*(y1-y0);
	return areaInt/((float)Area()+(float)rOther.Area()-areaInt);
}

template <typename T>
template <typename T2>
bool RettBoundingBox<T>::Interno(const RettBoundingBox<T2> &rOther) const
{
	return (XMin()>=rOther.XMin()) && (YMin()>=rOther.YMin()) && (XMax()<=rOther.XMax()) && (YMax()<=rOther.YMax());
}

#endif
