/* 
 * Header file della classe Tracker
 */

#ifndef TRACKER_H
#define TRACKER_H

#include "RettBoundingBox.h"

#include <vector>
#include <../Eigen/Eigen/Core>
#include <opencv/cv.h>

class FileConfigurazione;
class Features;
class Kernel;
class OnlineLearning;
class Immagine;

class Tracker
{
public:
	Tracker(const FileConfigurazione & conf);
	~Tracker();
	
	void Initialise(const cv::Mat& frame, FloatRect bb);
	void Reset();
	void Track(const cv::Mat& frame);
	void Debug();
	double GetFeaturesVector(int index);
	inline const FloatRect& GetBB() const { return m_bb; }
	inline bool Inizializzato() const { return Mio_Inizializzato; }
	
private:
	const FileConfigurazione &Mia_Configurazione;
	bool Mio_Inizializzato;
	std::vector<Features*> Mie_Features;
	std::vector<Kernel*> m_kernels;
	OnlineLearning *Mio_PLearner;
	FloatRect m_bb;
	cv::Mat Mia_ImmagineDebug;
	bool ServeImmagineIntegrale;
	bool m_needsIntegralHist;


	void AggiornaLearner(const Immagine &image);
	void AggiornaImmagineDebug(const std::vector<FloatRect> &samples, const FloatRect &centre,
							   const std::vector<double> &scores);
};

#endif
