
#include "Tracker.h"
#include "FileConfigurazione.h"
#include <thread>
#include <fstream>
#include <opencv2/opencv.hpp>
// Include TraX library header
#include <trax.h>
#include <trax/client.hpp>
#include <opencv.hpp>

#define NUM_TRACKERS 5
#define HAAR_SIZE 192

using namespace std;
using namespace cv;

static const int LarghezzaBBCamera = 150;
static const int AltezzaBBCamera = 150;

vector<Tracker *> trackers;

void Rettangolo(Mat &rMat, const FloatRect &rRect, const Scalar &rColour) {
    IntRect r(rRect);
    rectangle(rMat, Point(r.XMin(), r.YMin()), Point(r.XMax(), r.YMax()), rColour);
}


float cosine_similarity(double *A, double *B, unsigned int lenght)
{
    float dot = 0.0, denom_a = 0.0, denom_b = 0.0 ;

    for(unsigned short i = 0; i < lenght; ++i) {
        dot += A[i] * B[i] ;
        denom_a += A[i] * A[i] ;
        denom_b += B[i] * B[i] ;
    }
    return dot / (sqrt(denom_a) * sqrt(denom_b));
}

int main(int argc, char *argv[]) {
	
    // reading config file
    string configPath = "../config.txt";
    int tid;
    int n_superpixel = 20;
	int frameInd = 0;

    vector<Rect> Patches; // Patches 
    Patches.resize(NUM_TRACKERS);

    vector<Rect> Patches_old; // old patches
    Patches_old.resize(NUM_TRACKERS);

    if (argc > 1) {
        configPath = argv[1];
    }

    // parsing config.txt
    FileConfigurazione conf(configPath);
    //cout << conf << endl;

    if (conf.TipologiaFeatures.size() == 0) {
        cout << "error: no features specified in config" << endl;
        return EXIT_FAILURE;
    }

    ofstream outFile;
    if (conf.resultsPath != "") {
        outFile.open(conf.resultsPath.c_str(), ios::out);
        if (!outFile) {
            cout << "error: could not open results file: " << conf.resultsPath << endl;
            return EXIT_FAILURE;
        }
    }

    // using camera if no sequence name is set
    bool useCamera = (conf.sequenceName == "");

    VideoCapture cap;
    bool reinit = false; 
    int startFrame = 0;
    int endFrame = -1;
    FloatRect initBB,bb_re;
    //local trackers bounding boxes...s
    vector<FloatRect> m_bb_iniziali(NUM_TRACKERS);
    string imgFormat;
    float scaleW = 1.f;
    float scaleH = 1.f;
    m_bb_iniziali.clear();

    imgFormat = conf.sequenceBasePath + "/" + conf.sequenceName + "/%08d.jpg";

    scaleW = 1.0;
    scaleH = 1.0;

    for (int i = 0; i < NUM_TRACKERS; i++) {
        trackers.push_back(new Tracker(conf));
    }

    if (!conf.quietMode) {
        namedWindow("result");
    }

    Mat result(conf.frameHeight, conf.frameWidth, CV_8UC3);
    bool paused = false;
    bool doInitialise = false;
    bool deltaDelete = false;
    double mat_vectors_i[NUM_TRACKERS][HAAR_SIZE];//Matrice dei vettori al passo i

    srand(time(NULL));//reinizializzato il seed per i numeri pseudocasuali
    int out_track=-1;
    int trackers_in[NUM_TRACKERS];

    float xmin = -1.f;
    float ymin = -1.f;
    float width = -1.f;
    float height = -1.f;
    
    trax::Server handle(trax::Metadata(TRAX_REGION_RECTANGLE,TRAX_IMAGE_PATH | TRAX_IMAGE_MEMORY | TRAX_IMAGE_BUFFER),trax_no_log);

    while (true) {  // VOT

        Mat frame;
        trax::Region region;
        trax::Image image;
        trax::Properties properties;

        int tr = handle.wait(image, region, properties);
        if(tr == TRAX_INITIALIZE) {
            cout<<"Initialized\n";
            scaleW = 1.0;
            scaleH = 1.0;

            xmin = trax::region_to_rect(region).x;
            ymin = trax::region_to_rect(region).y;
            width = trax::region_to_rect(region).width;
            height = trax::region_to_rect(region).height;
            cout<<"xmin: "<<xmin <<endl;
            cout<<"ymin: "<<ymin <<endl;
            cout<<"width: "<<width <<endl;
            cout<<"height: "<<height <<endl;
            bb_re = FloatRect(xmin * scaleW, ymin * scaleH, width * scaleW, height * scaleH);
            //Inizializzo i Bounding Box per i 4 tracker...s
            m_bb_iniziali[0] = FloatRect(xmin * scaleW, ymin * scaleH, width / 2 * scaleW + width / 4 , height / 2 * scaleH + height / 4);
            m_bb_iniziali[1] = FloatRect((xmin + (width / 2) - (width / 4)) * scaleW, ymin * scaleH, width / 2  * scaleW + width / 4,
                                         height / 2 * scaleH + height / 4);
            m_bb_iniziali[2] = FloatRect(xmin * scaleW, (ymin + (height / 2)-(height / 4)) * scaleH, width / 2 * scaleW+ width / 4,
                                         height / 2 * scaleH + height / 4);
            m_bb_iniziali[3] = FloatRect((xmin + (width / 2)-(width / 4)) * scaleW, (ymin + (height / 2)-(height / 4)) * scaleH, width / 2 * scaleW+ width / 4,
                                         height / 2 * scaleH+ height / 4);
            m_bb_iniziali[4] = FloatRect((xmin + (width / 4)) - (width/8) * scaleW, (ymin + (height / 4))- (height/8) * scaleH, width / 2  * scaleW+ width / 4,
                                         height / 2 * scaleH + height / 4);
            region = trax::rect_to_region(Rect((int)xmin,(int)ymin,(int)width,(int)height));
            handle.reply(region,trax::Properties());
            

        } else if(tr == TRAX_FRAME ) {
            Mat frameOrig = trax::image_to_mat(image);

            frame = Mat(frameOrig);

            if (frameInd == 1) {
        
                Rect roi_i;
                
                #pragma omp parallel num_threads(NUM_TRACKERS) shared(Patches_old,m_bb_iniziali,n_superpixel) private(tid,roi_i)
                {
                    tid = omp_get_thread_num();
                    roi_i = Rect((int)m_bb_iniziali[tid].XMin(),(int)m_bb_iniziali[tid].YMin(),(int)(m_bb_iniziali[tid].XMax()-m_bb_iniziali[tid].XMin()),(int)(m_bb_iniziali[tid].YMax()-m_bb_iniziali[tid].YMin()));
                    Patches_old[tid] = roi_i;
                }
                //Initializing trackers
                #pragma omp parallel num_threads(NUM_TRACKERS) shared(trackers,frame,m_bb_iniziali) private(tid)
                {
                    tid = omp_get_thread_num();
                    trackers[tid]->Initialise(frame,m_bb_iniziali[tid]);
                }
                
            }
             
            // reinitialised tracker
            if (trackers[0]->Inizializzato() || trackers[1]->Inizializzato() ||trackers[2]->Inizializzato()||trackers[3]->Inizializzato()||trackers[4]->Inizializzato()) {

                #pragma omp parallel num_threads(NUM_TRACKERS) shared(frame) private(tid)
                {
                    tid = omp_get_thread_num();
                    trackers[tid]->Track(frame);
                }

                vector<FloatRect> bb_calcolati(NUM_TRACKERS);
                vector<int> x_centroide(NUM_TRACKERS);
                vector<int> y_centroide(NUM_TRACKERS);

                // Getting estimated bounding boxes
                #pragma omp parallel num_threads(NUM_TRACKERS) shared(bb_calcolati) private(tid)
                {
                    tid = omp_get_thread_num();
                    bb_calcolati[tid] = trackers[tid]->GetBB();
                }

                int j;


                // ********************************************************************** outlier detection **********************************************************

                bool corr_cancella=false;
                float norm_T = 0.0;
                float similarityMatrix[NUM_TRACKERS][NUM_TRACKERS];
                float means[NUM_TRACKERS];
                vector<int> x_centroide_clus(2);
                vector<int> y_centroide_clus(2);
                double max_dist = 1.0;
                int max_dist_index = -1;
                int elim=0;
                if(frameInd!=0)
                {
                    int i,j,tid;
                    Rect roi_i;
                    
                    vector<Point2f> Delta;
                    Delta.resize(NUM_TRACKERS);
                    #pragma omp parallel num_threads(NUM_TRACKERS) shared(trackers,Patches,Patches_old,frame,Delta) private(tid,roi_i)
                    {
                        tid = omp_get_thread_num();
                        roi_i = Rect((int)trackers[tid]->GetBB().XMin(),(int)trackers[tid]->GetBB().YMin(),(int)(trackers[tid]->GetBB().XMax() - trackers[tid]->GetBB().XMin()),(int)(trackers[tid]->GetBB().YMax()-trackers[tid]->GetBB().YMin()));
                        Patches[tid] = roi_i;
                        // Computing Deltax and Deltay
                        Delta[tid].x = Patches[tid].x - Patches_old[tid].x;
                        Delta[tid].y = Patches[tid].y - Patches_old[tid].y;
                    }

                    double max_dist=0;
                    double soglia=0;
                    double dist;
                    for(int i = 0; i < NUM_TRACKERS;i++)
                    {
                        double media1_x=0;
                        double media1_y=0;
                        for(int j=0;j<NUM_TRACKERS;j++)
                        {


                            if(j!=i){ // Sum x and y
                                media1_x += Delta[j].x;
                                media1_y += Delta[j].y;
                            }
                            media1_x=media1_x/(NUM_TRACKERS-1);
                            media1_y=media1_y/(NUM_TRACKERS-1);
                            dist=sqrt(pow(media1_x-Delta[i].x,2)+pow(media1_y-Delta[i].y,2));
                            if(abs(dist)>max_dist)
                            {
                                max_dist_index=i;
                                max_dist=dist;
                                soglia=max(media1_x,media1_y);
                            }
                            //soglia = sqrt(pow(media1_x,2) + pow(media1_y,2));
                        }

                    }
                    if (abs(max_dist)>0.5*sqrt(pow(initBB.XMax() - initBB.XMin(),2) + pow(initBB.YMax() - initBB.YMin(),2)))
                    {
                        bb_calcolati.erase(bb_calcolati.begin() + max_dist_index);
                        reinit = true;
                        trackers_in[max_dist_index]=0;
                    }

                    for(int k=0;k<bb_calcolati.size();k++)
                        if (abs(Delta[k].x)<1 &&abs(Delta[k].y)<1)
                        {
                            bb_calcolati.erase(bb_calcolati.begin() + k);
                            reinit = true;
                            trackers_in[k]=0;
                        }
                    // convex Hull
                }
                unsigned long offset= bb_calcolati.size();
                //cout<<endl<<"size bb calcolati"<<bb_calcolati.size()<<endl;
                vector<int> x_bb(2*offset);
                vector<int> y_bb(2*offset);

                for (int i = 0; i < offset;i++)
                { // Convex Hull - looping over BBes
                    x_bb[i] = (int) bb_calcolati[i].XMin();
                    x_bb[i+offset] = (int) bb_calcolati[i].XMax();
                    y_bb[i] = (int) bb_calcolati[i].YMin();
                    y_bb[i+offset] = (int) bb_calcolati[i].YMax();
                }


                //min,max (x,y)
                auto result_x = minmax_element(x_bb.begin(), x_bb.end());
                auto result_y = minmax_element(y_bb.begin(), y_bb.end());
                
                initBB.Set(x_bb[(result_x.first - x_bb.begin())], y_bb[(result_y.first - y_bb.begin())], x_bb[(result_x.second - x_bb.begin())] - x_bb[(result_x.first - x_bb.begin())], y_bb[(result_y.second - y_bb.begin())] - y_bb[(result_y.first - y_bb.begin())]);
                
                float out_xmin=initBB.XMin();
                float out_ymin=initBB.YMin();
                float out_height=initBB.Height();
                float out_width=initBB.Width();
                
                Rect rect=Rect((int) out_xmin, (int) out_ymin, (int) out_width, (int) out_height);
                if(reinit){
                    
                    for(int i=0;i<NUM_TRACKERS;i++)
                    {
                        if(trackers_in[i]==0)
                        {
                            FloatRect bb_init = FloatRect(xmin+width/2-(trackers[i]->GetBB().Width())/2 ,ymin+height/2-(trackers[i]->GetBB().Height())/2,trackers[i]->GetBB().Width(),trackers[i]->GetBB().Height());//Tracker TOP SX
                            trackers[i]->Initialise(frame,bb_init);
                            trackers_in[i]=1;
                        }
                    }
                    reinit = false;
                    
                }

                handle.reply( trax::rect_to_region(rect),trax::Properties());
                Patches_old = Patches;
                frameInd++;

                if(reinit)
                {
                    bb_re=initBB;
                }

            }
        } else
            break;

        frameInd++;
    }
    if (outFile.is_open()) {
        outFile.close();
    }
    exit(0);
}
